import unittest
import docker

from tests.utils import run_check
from tests.utils.ssh_test import SSHServerContainerRequirement


class SshCommandCheckTest(SSHServerContainerRequirement, unittest.TestCase):
    docker_client: docker.DockerClient

    def test_not_passed_host_raises_human_readable_message(self):
        result = run_check('ssh-files-checksum', {}, {})

        self.assertIn('HOST is mandatory', result.output.strip())
        self.assertFalse(result.exit_status)

    def test_success_case(self):
        result = run_check('ssh-files-checksum', {
            'HOST': '127.0.0.1',
            'PORT': 3223,
            'USER': 'root',
            'PASSWORD': 'root',
            'SSH_OPTS': '-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null',
            'EXPECTS': {
                "/dev/null": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"
            }
        }, {})

        self.assertEqual('All checksums are matching', result.output.strip())
        self.assertTrue(result.exit_status)

    def test_at_least_one_checksum_not_matching(self):
        result = run_check('ssh-files-checksum', {
            'HOST': '127.0.0.1',
            'PORT': 3223,
            'USER': 'root',
            'PASSWORD': 'root',
            'SSH_OPTS': '-o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null',
            'EXPECTS': {
                "/dev/null": "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
                "/bin/sh": "will-not-match-this-one"
            }
        }, {})

        self.assertIn("FAIL: '/bin/sh' checksum is not matching. Expected: 'will-not-match-this-one'", result.output.strip())
        self.assertFalse(result.exit_status)



#!/usr/bin/python3

import os
import warnings
import inspect
from unittest import TestCase
from testcontainers.core.container import DockerContainer
from testcontainers.core.waiting_utils import wait_for_logs

path = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))) + '/../infracheck/checks/'
TlsDockerNetworkCheck: any
filename = path + '/tls-docker-network'
exec(compile(open(filename, "rb").read(), filename, 'exec'))


class FunctionalTestTlsDockerNetwork(TestCase):
    def setUp(self):
        warnings.simplefilter("ignore")

    def test_multiple_containers_with_multiple_domains_per_container(self):
        """
        There are two containers. Matching is by environment variable INFR_VIRTUAL_HOST={{ domain }}
        One with single domain: google.com
        Second one: duckduck.com and bing.com

        Checks:
            - There is a correct connection to docker, and proper parsing of the data from docker
            - "tls" check is correctly called
        """

        first = DockerContainer(image='nginx:1.19').with_env('INFR_VIRTUAL_HOST', 'duckduckgo.com,bing.com').start()
        second = DockerContainer(image='nginx:1.19').with_env('INFR_VIRTUAL_HOST', 'google.com').start()

        try:
            wait_for_logs(first, 'ready for start up')
            wait_for_logs(second, 'ready for start up')

            os.environ['PATH'] = path + ':' + os.environ['PATH']
            check = TlsDockerNetworkCheck(param_type='environment', param_name='INFR_VIRTUAL_HOST', alert_days_before=1)\
                .main()

        finally:
            first.stop()
            second.stop()

        self.assertIn('Domain google.com is OK', check[0])
        self.assertIn('Domain bing.com is OK', check[0])
        self.assertIn('Domain duckduckgo.com is OK', check[0])
        self.assertTrue(check[1])

    def test_containers_matched_by_label(self):
        """
        There are two docker containers. Matching is by label org.riotkit.domain: {{ domain }}

        Checks:
            - Parsing of the labels syntax
        """

        first = DockerContainer(image='nginx:1.19').with_kwargs(labels={'org.riotkit.domain': 'duckduckgo.com'}).start()
        second = DockerContainer(image='nginx:1.19')\
            .with_kwargs(labels={'org.riotkit.domain': 'riseup.net,bing.com'}).start()

        try:
            wait_for_logs(first, 'ready for start up')
            wait_for_logs(second, 'ready for start up')

            os.environ['PATH'] = path + ':' + os.environ['PATH']
            check = TlsDockerNetworkCheck(param_type='label', param_name='org.riotkit.domain', alert_days_before=1) \
                .main()

        finally:
            first.stop()
            second.stop()

        self.assertIn('Domain duckduckgo.com is OK', check[0])
        self.assertIn('Domain bing.com is OK', check[0])
        self.assertIn('Domain riseup.net is OK', check[0])
        self.assertTrue(check[1])

    def test_no_domains_found_will_result_in_error_status(self):
        check = TlsDockerNetworkCheck(param_type='label', param_name='invalid-label-name', alert_days_before=1) \
            .main()

        self.assertEqual('No any domains found, maybe the containers are not running?', check[0])
        self.assertFalse(check[1])




#!/usr/bin/python3

import smtplib
from typing import Tuple
from unittest import TestCase

import infracheck.checks.smtp_credentials_check as smtpcheck


class UnitTestSmtpCredentials(TestCase):
    @staticmethod
    def _create_mocked_method_raising(exception: Exception = None):
        def mock(*args, **kwargs):
            if exception == None:
                pass
            else:
                raise exception

        check = smtpcheck.SMTPCheck()
        check._verify_connection = mock

        return check

    @staticmethod
    def _run_mocked_check(testname: str, check: smtpcheck.SMTPCheck) \
            -> Tuple[bool, str]:

        status, txt, err = check.main("host", 25, "username", "password", '', 1)

        return status, txt

    def test_expecting_success(self) -> None:
        check = self._create_mocked_method_raising()
        status, message = self._run_mocked_check(self.test_expecting_success.__name__, check)

        self.assertEqual((status, message), (True, smtpcheck.Messages.SUCCESS.value))

    def test_expecting_SMTP_connection_error(self) -> None:
        check = UnitTestSmtpCredentials._create_mocked_method_raising(smtplib.SMTPConnectError(0, ''))

        status, message = UnitTestSmtpCredentials._run_mocked_check(
            self.test_expecting_SMTP_connection_error.__name__, check
        )

        self.assertEqual((status, message), (False, smtpcheck.Messages.SMTP_CONNECTION_ERROR.value))

    def test_expecting_general_connection_refused_error(self) -> None:
        check = UnitTestSmtpCredentials._create_mocked_method_raising(ConnectionRefusedError())
        status, message = UnitTestSmtpCredentials._run_mocked_check(self.test_expecting_general_connection_refused_error.__name__, check)

        self.assertEqual((status, message), (False, smtpcheck.Messages.CONNECTION_REFUSED_ERROR.value))

    def test_expecting_SMTP_helo_error(self) -> None:
        check = UnitTestSmtpCredentials._create_mocked_method_raising(smtplib.SMTPHeloError(0, ''))
        status, message = UnitTestSmtpCredentials._run_mocked_check(self.test_expecting_SMTP_helo_error.__name__, check)

        self.assertEqual((status, message), (False, smtpcheck.Messages.SMTP_HELO_OR_EHLO_ERROR.value))

    def test_expecting_SMTP_server_disconnected_error(self) -> None:
        check = UnitTestSmtpCredentials._create_mocked_method_raising(smtplib.SMTPServerDisconnected())
        status, message = UnitTestSmtpCredentials._run_mocked_check(
            self.test_expecting_SMTP_server_disconnected_error.__name__,
            check
        )

        self.assertEqual((status, message), (False, smtpcheck.Messages.SMTP_DISCONNECTION_ERROR.value))

    def test_expecting_SMTP_authentication_error(self) -> None:
        check = UnitTestSmtpCredentials._create_mocked_method_raising(smtplib.SMTPAuthenticationError(0, ''))
        status, message = UnitTestSmtpCredentials._run_mocked_check(
            self.test_expecting_SMTP_authentication_error.__name__,
            check
        )

        self.assertEqual((status, message), (False, smtpcheck.Messages.SMTP_AUTHENTICATION_ERROR.value))

    def test_expecting_general_SMTP_error(self) -> None:
        check = UnitTestSmtpCredentials._create_mocked_method_raising(smtplib.SMTPException())
        status, message = UnitTestSmtpCredentials._run_mocked_check(
            self.test_expecting_general_SMTP_error.__name__,
            check
        )

        self.assertEqual((status, message), (False, smtpcheck.Messages.GENERAL_SMTP_ERROR.value))

    def test_expecting_general_error(self) -> None:
        check = UnitTestSmtpCredentials._create_mocked_method_raising(Exception())
        status, message = UnitTestSmtpCredentials._run_mocked_check(self.test_expecting_general_error.__name__, check)

        self.assertEqual((status, message), (False, smtpcheck.Messages.GENERAL_ERROR.value))

    def test_expecting_auth_command_not_supported_error(self) -> None:
        check = UnitTestSmtpCredentials._create_mocked_method_raising(smtplib.SMTPNotSupportedError())

        status, message = UnitTestSmtpCredentials._run_mocked_check(
            self.test_expecting_auth_command_not_supported_error.__name__,
            check
        )

        self.assertEqual(
            (status, message),
            (False, smtpcheck.Messages.SMTP_AUTH_METHOD_NOT_SUPPORTED_BY_SERVER.value)
        )



#!/usr/bin/python3

import os
import inspect
from unittest import TestCase

path = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))) + '/../infracheck/checks/'
TlsCheck: any
filename = path + '/tls'
exec(compile(open(filename, "rb").read(), filename, 'exec'))


class UnitTestTlsCheck(TestCase):
    def test_certificate_is_expiring(self) -> None:
        check = TlsCheck()
        check.get_certificate_expiration_date = lambda domain, host, port: ("Jul 31 23:59:59 1990 GMT", "...")

        status, message = check.main(domain='', host='', port=443, alert_days_before=1)
        self.assertIn('Certificate is expiring soon, only', message)
        self.assertFalse(status)

    def test_certificate_not_expired(self) -> None:
        check = TlsCheck()
        check.get_certificate_expiration_date = lambda domain, host, port: ("Jul 31 23:59:59 2999 GMT", "...")

        status, message = check.main(domain='', host='', port=443, alert_days_before=1)
        self.assertTrue(status)
        self.assertEqual('Certificate valid until Jul 31 23:59:59 2999 GMT', message)

    def test_certificate_invalid_format_returned_by_openssl(self) -> None:
        check = TlsCheck()
        check.get_certificate_expiration_date = lambda domain, host, port: ("sometime", "...")

        status, message = check.main(domain='', host='', port=443, alert_days_before=1)
        self.assertFalse(status)
        self.assertIn("Error parsing date - invalid format returned by OpenSSL binary 'sometime'", message)

    def test_invalid_openssl_output_is_catch(self) -> None:
        check = TlsCheck()

        status, message = check.main(domain='localhost', host='localhost', port=9999, alert_days_before=1)
        self.assertIn('Error parsing date - invalid format returned by OpenSSL binary', message)



#!/usr/bin/python3

import os
import warnings
from unittest import TestCase
from testcontainers.core.container import DockerContainer
from testcontainers.core.waiting_utils import wait_for_logs

import infracheck.checks.smtp_credentials_check as smtpcheck


class FunctionalTestSmtpCredentials(TestCase):
    """
    SMTP functional tests using a real Postfix container
    ----------------------------------------------------
    Check container docs for detailed usage: https://github.com/riotkit-org/smtp-ext-relay
    """

    container: DockerContainer = None

    def tearDown(self):
        if self.container:
            self.container.stop()

    @staticmethod
    def _create_container() -> DockerContainer:
        return DockerContainer(image='quay.io/riotkit/smtp:v.1.1.0')\
            .with_command('bakunin.example.org durruti:durruti')\
            .with_bind_ports(25, 2525)

    def _start_container(self, container: DockerContainer):
        warnings.simplefilter("ignore")
        self.container = container
        self.container.start()
        wait_for_logs(self.container, 'daemon started')

    def test_connection_without_encryption_success_when_credentials_are_correct(self):
        self._start_container(self._create_container().with_env('SMTP_USE_TLS', 'false'))

        status, txt, err = smtpcheck.SMTPCheck().main("localhost", 2525, "durruti", "durruti", '', 15)

        self.assertTrue(status)
        self.assertEqual('Success', txt)

    def test_connection_without_encryption_failures_when_credentials_are_invalid(self):
        self._start_container(self._create_container().with_env('SMTP_USE_TLS', 'false'))

        status, txt, err = smtpcheck.SMTPCheck().main("localhost", 2525, "invalid", "invalid", '', 5)

        self.assertFalse(status)
        self.assertEqual('Authentication failed', txt)

    def test_connects_to_tls(self):
        self._start_container(
            self._create_container()
                .with_env('SMTP_USE_TLS', 'true')
                .with_env('SMTPD_TLS_CERT_FILE', '/keys/bakunin.example.org.crt')
                .with_env('SMTPD_TLS_KEY_FILE', '/keys/bakunin.example.org.key')
                .with_env('SMTP_TLS_CA_FILE', '/keys/cacert.pem')
                .with_env('ENABLE_DKIM', 'false')
                .with_volume_mapping(os.path.dirname(os.path.realpath(__file__)) + '/files/keys', '/keys', mode='ro')
        )

        status, txt, err = smtpcheck.SMTPCheck().main("127.0.0.1", 2525, "durruti", "durruti", 'starttls_self_signed', 5)
        self.assertTrue(status)

    def test_does_not_connect_to_starttls_if_self_signed_certificate_is_not_allowed(self):
        self._start_container(
            self._create_container()
                .with_env('SMTP_USE_TLS', 'true')
                .with_env('SMTPD_TLS_CERT_FILE', '/keys/bakunin.example.org.crt')
                .with_env('SMTPD_TLS_KEY_FILE', '/keys/bakunin.example.org.key')
                .with_env('SMTP_TLS_CA_FILE', '/keys/cacert.pem')
                .with_env('ENABLE_DKIM', 'false')
                .with_volume_mapping(os.path.dirname(os.path.realpath(__file__)) + '/files/keys', '/keys', mode='ro')
        )

        status, txt, err = smtpcheck.SMTPCheck().main("127.0.0.1", 2525, "durruti", "durruti", 'starttls', 5)
        self.assertIn('[SSL: CERTIFICATE_VERIFY_FAILED] certificate verify failed', str(err))
        self.assertFalse(status)

    def test_connection_refused_when_invalid_port_entered(self):
        status, txt, err = smtpcheck.SMTPCheck().main("127.0.0.1", 2529, "durruti", "durruti", '', 2)
        self.assertFalse(status)
        self.assertEqual('Connection refused', txt)



#!/usr/bin/python3
"""
<sphinx>
smtp_credentials_check.py
-------------------------

Verifies connection, TLS certificate and credentials to a SMTP server by doing a ping + authorization try.

Parameters:

- smtp_host (example: bakunin.example.org)
- smtp_port (example: 25)
- smtp_user (example: noreply@example.org)
- smtp_password (example: bakunin-1936)
- smtp_encryption (example: starttls. Values: "", "ssl", "starttls")
- smtp_timeout (default: 30, unit: seconds)
</sphinx>
"""

import enum
import os
import smtplib
import ssl
import socket
import sys
from typing import Tuple, Optional


class Messages(enum.Enum):
    SUCCESS = 'Success'
    SMTP_CONNECTION_ERROR = 'Error connecting to the SMTP server'
    SMTP_HELO_OR_EHLO_ERROR = 'HELO or EHLO error'
    SMTP_AUTHENTICATION_ERROR = 'Authentication failed'
    SMTP_DISCONNECTION_ERROR = 'Server is disconnected'
    GENERAL_ERROR = 'General error'
    GENERAL_SMTP_ERROR = 'General SMTP error'
    SOCKET_TIMEOUT = 'Connection timeout'
    CONNECTION_REFUSED_ERROR = 'Connection refused'
    SMTP_AUTH_METHOD_NOT_SUPPORTED_BY_SERVER = 'AUTH command not supported by the server'


class EnvKeys(enum.Enum):
    HOST = 'SMTP_HOST'
    PORT = 'SMTP_PORT'
    USERNAME = 'SMTP_USER'
    PASSWORD = 'SMTP_PASSWORD'
    ENCRYPTION = 'SMTP_ENCRYPTION'
    TIMEOUT = 'SMTP_TIMEOUT'
    DEBUG = 'SMTP_DEBUG'


class Encryption(enum.Enum):
    ENC_NONE = ''
    ENC_SSL = 'ssl'
    ENC_STARTTLS = 'starttls'
    ENC_STARTTLS_SELF_SIGNED = 'starttls_self_signed'


class SMTPCheck(object):
    def main(self, host: str, port: int, username: str, password: str, encryption: str,
             timeout: int, debug: bool = False) \
            -> Tuple[bool, str, Optional[Exception]]:

        try:
            self._verify_connection(host, port, username, password, encryption, timeout, debug)
            return True, Messages.SUCCESS.value, None

        except smtplib.SMTPConnectError as exc:
            return False, Messages.SMTP_CONNECTION_ERROR.value, exc

        except smtplib.SMTPHeloError as exc:
            return False, Messages.SMTP_HELO_OR_EHLO_ERROR.value, exc

        except smtplib.SMTPAuthenticationError as exc:
            return False, Messages.SMTP_AUTHENTICATION_ERROR.value, exc

        except smtplib.SMTPServerDisconnected as exc:
            return False, Messages.SMTP_DISCONNECTION_ERROR.value, exc

        except smtplib.SMTPNotSupportedError as exc:
            return False, Messages.SMTP_AUTH_METHOD_NOT_SUPPORTED_BY_SERVER.value, exc

        except smtplib.SMTPException as exc:
            return False, Messages.GENERAL_SMTP_ERROR.value, exc

        except socket.timeout as exc:
            return False, Messages.SOCKET_TIMEOUT.value, exc

        except ConnectionRefusedError as exc:
            return False, Messages.CONNECTION_REFUSED_ERROR.value, exc

        except Exception as exc:
            return False, Messages.GENERAL_ERROR.value, exc

    @staticmethod
    def _verify_connection(host: str, port: int, username: str, password: str,
                           encryption: str, timeout: int, debug: bool):

        connection = smtplib.SMTP_SSL(host, port, timeout=timeout) if encryption == Encryption.ENC_SSL.value else \
            smtplib.SMTP(host, port, timeout=timeout)

        connection.set_debuglevel(1 if debug else 0)

        if encryption == Encryption.ENC_STARTTLS.value:
            connection.starttls(context=ssl.create_default_context())

        elif encryption == Encryption.ENC_STARTTLS_SELF_SIGNED.value:
            connection.starttls(context=ssl._create_unverified_context())

        if username and password:
            connection.login(username, password)

        connection.close()


if __name__ == '__main__':
    inputs = {
        EnvKeys.HOST.value: os.getenv(EnvKeys.HOST.value),
        EnvKeys.PORT.value: os.getenv(EnvKeys.PORT.value),
        EnvKeys.USERNAME.value: os.getenv(EnvKeys.USERNAME.value),
        EnvKeys.PASSWORD.value: os.getenv(EnvKeys.PASSWORD.value),
        EnvKeys.ENCRYPTION.value: os.getenv(EnvKeys.ENCRYPTION.value, 'starttls'),
        EnvKeys.TIMEOUT.value: os.getenv(EnvKeys.TIMEOUT.value, 30),
        EnvKeys.DEBUG.value: os.getenv(EnvKeys.DEBUG.value, False)
    }

    for key in inputs:
        if inputs[key] is None:
            print('Missing environment variable: {}'.format(key))
            sys.exit(1)

    app = SMTPCheck()
    is_success, message, exception = app.main(
        inputs[EnvKeys.HOST.value],
        int(inputs[EnvKeys.PORT.value]),
        inputs[EnvKeys.USERNAME.value],
        inputs[EnvKeys.PASSWORD.value],
        inputs[EnvKeys.ENCRYPTION.value],
        int(inputs[EnvKeys.TIMEOUT.value]),
        bool(inputs[EnvKeys.DEBUG.value])
    )

    print(('Error: {}'.format(message)) if exception else message)

    if exception is not None:
        print('Exception:', exception)

    sys.exit(0 if is_success else 1)



import unittest
import os
import inspect
import unittest.mock
import datetime
import whois

path = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))) + '/../infracheck/checks/'
DomainCheck: any
filename = path + '/domain-expiration'
exec(compile(open(filename, "rb").read(), filename, 'exec'))


class WhoIsMock:
    expiration_date: datetime

    def __init__(self, expiration_date: datetime):
        self.expiration_date = expiration_date


class DomainExpirationCheckTest(unittest.TestCase):
    def test_ok(self):
        check = DomainCheck('google.com', 25)

        with unittest.mock.patch.object(check, 'whois') as whois_mocked:
            whois_mocked.query.return_value = WhoIsMock(
                datetime.datetime.now() + datetime.timedelta(days=161)
            )
            result = check.perform_check()

        self.assertIn("is not expired", result[1])
        self.assertTrue(result[0])

    def test_not_registered(self):
        check = DomainCheck('dijweqjiqewjioqewjioqwejiqeij-not-exists.com', 25)
        result = check.perform_check()

        self.assertIn("Domain seems to be not registered", result[1])
        self.assertFalse(result[0])

    def test_expired(self):
        check = DomainCheck('google.com', 25)

        with unittest.mock.patch.object(check, 'whois') as whois_mocked:
            whois_mocked.query.return_value = WhoIsMock(datetime.datetime.now())

            result = check.perform_check()

        self.assertIn("Domain google.com expired", result[1])
        self.assertFalse(result[0])

    def test_sk_domain_manual_whois_parsing(self):
        check = DomainCheck('priamaakcia.sk', 25)

        def mock_library_raises_exception(*args, **kwargs):
            raise whois.exceptions.UnknownTld()

        check._ask_whois_library = mock_library_raises_exception
        check._ask_whois_in_shell = lambda *args, **kwargs: """
            Domain:                       priamaakcia.sk
            Registrant:                   WEBHOUSE-AZPNWEY
            Admin Contact:                WEBHOUSE
            Tech Contact:                 WEBHOUSE
            Registrar:                    IPEK-0001
            Created:                      2005-06-24
            Updated:                      2020-07-08
            Valid Until:                  2999-06-24
            Nameserver:                   ns1.afraid.org
            Nameserver:                   ns2.afraid.org
            EPP Status:                   ok
            
            Registrar:                    IPEK-0001
            Name:                         WebHouse, s.r.o.
            Organization:                 WebHouse, s.r.o.
            Organization ID:              36743852
            Phone:                        +421.332933500
            Email:                        domeny@webhouse.sk
            Street:                       Paulínska 20
            City:                         Trnava
            Postal Code:                  91701
            Country Code:                 SK
            Created:                      2017-09-01
            Updated:                      2021-05-17
            
            Contact:                      WEBHOUSE-AZPNWEY
            Registrar:                    IPEK-0001
            Created:                      2017-09-02
            Updated:                      2017-11-01
            
            Contact:                      WEBHOUSE
            Name:                         WebHouse, s. r. o.
            Organization:                 WebHouse, s. r. o.
            Organization ID:              36743852
            Phone:                        +421.910969601
            Email:                        domeny@webhouse.sk
            Street:                       Paulínska 20
            City:                         Trnava
            Postal Code:                  917 01
            Country Code:                 SK
            Registrar:                    IPEK-0001
            Created:                      2017-09-02
            Updated:                      2021-03-22
        """

        result = check.perform_check()

        self.assertIn('Domain priamaakcia.sk is not expired.', result[1])
        self.assertTrue(result[0])
